from distutils.core import setup
setup(
  name = 'tamber',
  packages = ['tamber'],
  version = '0.2.3',
  description = 'Tamber is the easiest way to put head-scratchingly accurate, real time recommendations in your app.  See https://tamber.com for details.',
  author = 'Tamber',
  author_email = 'team@tamber.com',
  url = 'https://github.com/tamber/tamber-python',
  download_url = 'https://pypi.python.org/pypi/tamber', 
  keywords = ['tamber', 'recommendations', 'pytamber', 'backend', 'hosted', 'cloud',
              'collaborative filtering', 'engine', 'suggestion engine', 
              'recommender system', 'suggestions'],
  classifiers = [
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ]
)